==> pot-0.1.0.tar.gz: lib/src/pot.dart <==
import 'package:meta/meta.dart';

import 'errors.dart';

part 'extensions.dart';

part 'pot_body.dart';

/// The signature of a callback that creates and returns an object of type T.
typedef PotObjectFactory<T> = T Function();

/// The signature of a callback that receives an object of type T to be
/// disposed of.
typedef PotDisposer<T> = void Function(T);

typedef _Resetter = void Function();
typedef _ScopedResetters = List<List<_Resetter>>;

/// A class that instantiate and keeps an object until it is discarded.
///
/// {@template pot.class}
/// This class is for keeping an object instance instead of assigning
/// it direct to a global variable.
///
/// When an object is first needed, it is created by the `factory` provided
/// to the constructor, and the `disposer` is triggered when the object
/// is discarded by methods such as [reset] and [ReplaceablePot.replace].
///
/// ```dart
/// final counterPot = Pot<Counter>(
///   () => Counter(),
///   disposer: (counter) => counter.dispose(),
/// );
///
/// void main() {
///   final counter = counterPot();
///   ...
///
///   // The object is discarded and the disposer function is called.
///   counterPot.reset();
/// }
/// ```
///
/// instead of:
///
/// ```dart
/// final counter = Counter();
/// ```
///
/// It allows a subtype to be used, making the object testable.
///
/// ```dart
/// final counterPot = Pot.replaceable<Counter>(() => Counter());
/// counterPot.replace(() => MockCounter());
/// ```
///
/// or if the pot is not one created by [Pot.replaceable]:
///
/// ```dart
/// Pot.forTesting = true;
/// final counterPot = Pot<Counter>(() => Counter());
/// counterPot.replaceForTesting(() => MockCounter());
/// ```
///
/// It also enables the object to be discarded when it is no longer
/// necessary.
///
/// ```dart
/// final counter = counterPot();
/// final repository = repositoryPot();
/// ...
/// Pot.resetAll();
/// ```
///
/// The [Pot] class also provides the feature of scoping the range
/// where particular objects exist.
///
/// ```dart
/// // The index of the current scope changes from 0 to 1.
/// Pot.pushScope();
///
/// // A counter object is created in the scope 1.
/// counterPot.create();
///
/// // The object is discarded when the scope 1 is removed.
/// Pot.popScope();
/// ```
/// {@endtemplate}
@sealed
class Pot<T> extends _PotBody<T> {
  /// Creates an object that instantiate and keeps an object until
  /// it is discarded.
  ///
  /// {@macro pot.class}
  Pot(PotObjectFactory<T> factory, {PotDisposer<T>? disposer})
      : super(factory, disposer: disposer);

  static int _currentScope = 0;
  static final _ScopedResetters _scopedResetters = [[]];

  @visibleForTesting
  // ignore: library_private_types_in_public_api
  static _ScopedResetters get $scopedResetters =>
      List.unmodifiable(<List<_Resetter>>[
        for (final resetter in _scopedResetters) List.unmodifiable(resetter),
      ]);

  /// The flag that shows whether [replaceForTesting] is enabled.
  ///
  /// Defaults to `false`, which means disabled.
  static bool forTesting = false;

  /// The index number of the current scope.
  ///
  /// The number starts from 0.
  ///
  /// {@macro pot.scope}
  ///
  /// ```dart
  /// void main() {
  ///   print(Pot.currentScope); // 0
  ///
  ///   Pot.pushScope();
  ///   print(Pot.currentScope); // 1
  ///
  ///   Pot.popScope();
  ///   print(Pot.currentScope); // 0
  /// }
  /// ```
  static int get currentScope => _currentScope;

  /// Creates a pot of type [ReplaceablePot] that has the ability
  /// to replace its factory.
  ///
  /// Replacing the factory resets the object created by the old
  /// factory. It should be done only when it is really necessary.
  /// In order to prevent unwanted replacement, it is only allowed
  /// in pots created by this method, and not in pots created by
  /// the default constructor.
  ///
  /// For details on how this method is used and what occurs in the
  /// process of replacement, see the document of [ReplaceablePot.replace].
  static ReplaceablePot<T> replaceable<T>(
    PotObjectFactory<T> factory, {
    PotDisposer<T>? disposer,
  }) {
    return ReplaceablePot<T>(factory, disposer: disposer);
  }

  /// Adds a new scope to the stack of scopes.
  ///
  /// {@template pot.scope}
  /// A "scope" here is a notion related to the lifespan of an object
  /// created by and held in a pot.
  ///
  /// For example, if the index number of the current scope is 2 and
  /// the factory set in the constructor is triggered for the first time,
  /// an object is created by the factory and gets bound to the scope 2.
  /// The object exists while the current scope is 2 or newer, so it is
  /// discarded when the scope 2 is removed.
  /// {@endtemplate}
  ///
  /// ```dart
  /// final counterPot = Pot<Counter>(() => Counter(0));
  ///
  /// void main() {
  ///   // A new scope is added, and the `currentScope` turns 1.
  ///   // The object is not bound to the scope yet because it hasn't created.
  ///   Pot.pushScope();
  ///
  ///   // An object is created and set in the current scope.
  ///   final counter = counterPot();
  ///
  ///   // The object is discarded.
  ///   // The scope 1 is removed and the `currentScope` turns 0.
  ///   Pot.popScope();
  /// }
  /// ```
  static void pushScope() {
    _scopedResetters.createScope();
    _currentScope++;
  }

  /// Removes the current scope from the stack of scopes.
  ///
  /// This resets all pots in the current scope, triggers the disposer of
  /// each of them, and decrements the index number of the current scope.
  ///
  /// ```dart
  /// final counterPot1 = Pot<Counter>(() => Counter(0));
  /// final counterPot2 = Pot<Counter>(() => Counter(0));
  ///
  /// void main() {
  ///   // A new scope is added, and the `currentScope` turns 1.
  ///   // The objects are not bound to the scope yet because
  ///   // they haven't created.
  ///   Pot.pushScope();
  ///
  ///   // Object are created and set in the current scope 1.
  ///   final counter1 = counterPot1();
  ///   final counter2 = counterPot2();
  ///
  ///   // The scope 1 is removed, and the objects in both
  ///   // of the two pots are discarded.
  ///   Pot.popScope();
  /// }
  /// ```
  ///
  /// If this is used in the root scope, the index number remains `0`
  /// although every pot in the scope is reset and its disposer is called.
  static void popScope() {
    _scopedResetters.clearScope(_currentScope);
    if (_currentScope > 0) _currentScope--;
  }

  /// Resets all pots in the current scope.
  ///
  /// This discards all the objects bound to the current scope, and
  /// triggers the disposer of each pot.
  ///
  /// Calling this does not affect the scope itself. The index number
  /// of the current scope stays the same.
  ///
  /// See [reset] for details on a reset of an object.
  static void resetAllInScope() {
    _scopedResetters.clearScope(_currentScope, keepScope: true);
  }

  /// Resets all pots of all scopes.
  ///
  /// This discards all the objects bound to any scopes, and
  /// triggers the disposer of each pot.
  ///
  /// Calling this does not affect the scopes themselves. The index
  /// number of the current scope stays the same.
  ///
  /// See [reset] for details on a reset of an object.
  static void resetAll() {
    for (var i = _currentScope; i >= 0; i--) {
      _scopedResetters.clearScope(i, keepScope: true);
    }
  }
}

==> pot-0.1.1.tar.gz: lib/src/pot.dart <==
import 'package:meta/meta.dart';

import 'errors.dart';

part 'extensions.dart';

part 'pot_body.dart';

/// The signature of a callback that creates and returns an object
/// of type [T].
typedef PotObjectFactory<T> = T Function();

/// The signature of a callback that receives an object of type [T]
/// to be disposed of.
typedef PotDisposer<T> = void Function(T);

typedef _Resetter = void Function();
typedef _ScopedResetters = List<List<_Resetter>>;

/// A class that instantiates and keeps an object of type [T] until
/// it is discarded.
///
/// {@template pot.class}
/// This class is for keeping an object instance instead of assigning
/// it direct to a global variable.
///
/// When an object is first needed, it is created by the `factory` provided
/// to the constructor, and the `disposer` is triggered when the object
/// is discarded by methods such as [reset] and [ReplaceablePot.replace].
///
/// ```dart
/// final counterPot = Pot<Counter>(
///   () => Counter(),
///   disposer: (counter) => counter.dispose(),
/// );
///
/// void main() {
///   final counter = counterPot();
///   ...
///
///   // The object is discarded and the disposer function is called.
///   counterPot.reset();
/// }
/// ```
///
/// instead of:
///
/// ```dart
/// final counter = Counter();
/// ```
///
/// It allows a subtype to be used, making the object testable.
///
/// ```dart
/// final counterPot = Pot.replaceable<Counter>(() => Counter());
/// counterPot.replace(() => MockCounter());
/// ```
///
/// or if the pot is not one created by [Pot.replaceable]:
///
/// ```dart
/// Pot.forTesting = true;
/// final counterPot = Pot<Counter>(() => Counter());
/// counterPot.replaceForTesting(() => MockCounter());
/// ```
///
/// It also enables the object to be discarded when it is no longer
/// necessary.
///
/// ```dart
/// final counter = counterPot();
/// final repository = repositoryPot();
/// ...
/// Pot.resetAll();
/// ```
///
/// The [Pot] class also provides the feature of scoping the range
/// where particular objects exist.
///
/// ```dart
/// // The index of the current scope changes from 0 to 1.
/// Pot.pushScope();
///
/// // A counter object is created in the scope 1.
/// counterPot.create();
///
/// // The object is discarded when the scope 1 is removed.
/// Pot.popScope();
/// ```
/// {@endtemplate}
@sealed
class Pot<T> extends _PotBody<T> {
  /// Creates a Pot that instantiates and keeps an object of type [T]
  /// until it is discarded.
  ///
  /// {@macro pot.class}
  Pot(PotObjectFactory<T> factory, {PotDisposer<T>? disposer})
      : super(factory, disposer: disposer);

  static int _currentScope = 0;
  static final _ScopedResetters _scopedResetters = [[]];

  @visibleForTesting
  // ignore: library_private_types_in_public_api
  static _ScopedResetters get $scopedResetters =>
      List.unmodifiable(<List<_Resetter>>[
        for (final resetter in _scopedResetters) List.unmodifiable(resetter),
      ]);

  /// The flag that shows whether [replaceForTesting] is enabled.
  ///
  /// Defaults to `false`, which means disabled.
  static bool forTesting = false;

  /// The index number of the current scope.
  ///
  /// The number starts from 0.
  ///
  /// {@macro pot.scope}
  ///
  /// ```dart
  /// void main() {
  ///   print(Pot.currentScope); // 0
  ///
  ///   Pot.pushScope();
  ///   print(Pot.currentScope); // 1
  ///
  ///   Pot.popScope();
  ///   print(Pot.currentScope); // 0
  /// }
  /// ```
  static int get currentScope => _currentScope;

  /// Creates a pot of type [ReplaceablePot] that has the ability
  /// to replace its factory.
  ///
  /// Replacing the factory resets the object created by the old
  /// factory. It should be done only when it is really necessary.
  /// In order to prevent unwanted replacement, it is only allowed
  /// in pots created by this method, and not in pots created by
  /// the default constructor.
  ///
  /// {@macro pot.replaceablePot}
  ///
  /// For details on replacement of the factory, see the document
  /// of [ReplaceablePot.replace].
  static ReplaceablePot<T> replaceable<T>(
    PotObjectFactory<T> factory, {
    PotDisposer<T>? disposer,
  }) {
    return ReplaceablePot<T>(factory, disposer: disposer);
  }

  /// Adds a new scope to the stack of scopes.
  ///
  /// {@template pot.scope}
  /// A "scope" here is a notion related to the lifespan of an object
  /// created by and held in a pot.
  ///
  /// For example, if the index number of the current scope is 2 and
  /// the factory set in the constructor is triggered for the first time,
  /// an object is created by the factory and gets bound to the scope 2.
  /// The object exists while the current scope is 2 or newer, so it is
  /// discarded when the scope 2 is removed.
  /// {@endtemplate}
  ///
  /// ```dart
  /// final counterPot = Pot<Counter>(() => Counter(0));
  ///
  /// void main() {
  ///   // A new scope is added, and the `currentScope` turns 1.
  ///   // The object is not bound to the scope yet because it hasn't created.
  ///   Pot.pushScope();
  ///
  ///   // An object is created and set in the current scope.
  ///   final counter = counterPot();
  ///
  ///   // The object is discarded.
  ///   // The scope 1 is removed and the `currentScope` turns 0.
  ///   Pot.popScope();
  /// }
  /// ```
  static void pushScope() {
    _scopedResetters.createScope();
    _currentScope++;
  }

  /// Removes the current scope from the stack of scopes.
  ///
  /// This resets all pots in the current scope, triggers the disposer of
  /// each of them, and decrements the index number of the current scope.
  ///
  /// ```dart
  /// final counterPot1 = Pot<Counter>(() => Counter(0));
  /// final counterPot2 = Pot<Counter>(() => Counter(0));
  ///
  /// void main() {
  ///   // A new scope is added, and the `currentScope` turns 1.
  ///   // The objects are not bound to the scope yet because
  ///   // they haven't created.
  ///   Pot.pushScope();
  ///
  ///   // Object are created and set in the current scope 1.
  ///   final counter1 = counterPot1();
  ///   final counter2 = counterPot2();
  ///
  ///   // The scope 1 is removed, and the objects in both
  ///   // of the two pots are discarded.
  ///   Pot.popScope();
  /// }
  /// ```
  ///
  /// If this is used in the root scope, the index number remains `0`
  /// although every pot in the scope is reset and its disposer is called.
  static void popScope() {
    _scopedResetters.clearScope(_currentScope);
    if (_currentScope > 0) _currentScope--;
  }

  /// Resets all pots in the current scope.
  ///
  /// This discards all the objects bound to the current scope, and
  /// triggers the disposer of each pot.
  ///
  /// Calling this does not affect the scope itself. The index number
  /// of the current scope stays the same.
  ///
  /// See [reset] for details on a reset of an object.
  static void resetAllInScope() {
    _scopedResetters.clearScope(_currentScope, keepScope: true);
  }

  /// Resets all pots of all scopes.
  ///
  /// This discards all the objects bound to any scopes, and
  /// triggers the disposer of each pot.
  ///
  /// Calling this does not affect the scopes themselves. The index
  /// number of the current scope stays the same.
  ///
  /// See [reset] for details on a reset of an object.
  static void resetAll() {
    for (var i = _currentScope; i >= 0; i--) {
      _scopedResetters.clearScope(i, keepScope: true);
    }
  }
}
